package controllers

import (
	"encoding/json"
	"net/http"
	"spaceofcheng/services/utils"

	"github.com/astaxie/beego"
)

type ControllerInterface interface {
	Constructor()
}

//BaseController 基础controller
type BaseController struct {
	beego.Controller
}

//Prepare 继承beego的
func (c *BaseController) Prepare() {
	c.initController()
}

// ResponseJSON 返回JSON数据
func (c *BaseController) ResponseJSON(data interface{}) {
	c.ResponseData(data, "", http.StatusOK)
}

// ResponseError 返回错误
func (c *BaseController) ResponseError(err error, code ...int) {
	if len(code) > 0 {
		c.ResponseData(nil, err, code[0])
	}

	c.ResponseData(nil, err, http.StatusBadRequest)
}

// ResponseRaw 返回
func (c *BaseController) ResponseRaw(msg []byte, stop ...bool) {
	c.Ctx.ResponseWriter.Write(msg)

	if stop != nil && len(stop) > 0 {
		if !stop[0] {
			return
		}
	}

	c.StopRun()
}

// ResponseData 自定义的JSON返回
func (c *BaseController) ResponseData(data interface{}, msg interface{}, code int) {
	status := code

	sendMessage := ""
	switch msgVal := msg.(type) {
	case error:
		sendMessage = msgVal.Error()
	case string:
		sendMessage = msgVal
	default:
		sendMessage = ""
	}

	c.Data["json"] = map[string]interface{}{
		"code":    status,
		"message": sendMessage,
		"result":  data,
	}

	c.Ctx.Output.Header("Access-Control-Expose-Headers", utils.TokenHeader)

	c.crosPolicy()
	c.ServeJSON()
	c.StopRun()
}

// initAppController 执行当前路由请求对应的 Controller 初始化
func (c *BaseController) initController() {
	if ctrl, ok := c.AppController.(ControllerInterface); ok {
		ctrl.Constructor()
	}
}

func (c *BaseController) GetBodyData() (map[string]interface{}, error) {
	var data map[string]interface{}
	err := json.Unmarshal(c.Ctx.Input.RequestBody, &data)
	return data, err
}

// Options 解决跨域先发送 options
func (c *BaseController) Options() {
	c.ResponseJSON("")
}

// crosPolicy 跨域策略
func (c *BaseController) crosPolicy() {
	runmode := beego.AppConfig.String("runmode")
	allowMode := beego.AppConfig.String("cros::allowMode")

	if runmode == allowMode {
		allowMethods := beego.AppConfig.String("cros::allowMethods")
		allowOrigin := beego.AppConfig.String("cros::allowOrigin")
		allowCredentials := beego.AppConfig.String("cros::allowCredentials")

		c.Ctx.Output.Header("Access-Control-Allow-Origin", allowOrigin)
		c.Ctx.Output.Header("Access-Control-Allow-Methods", allowMethods)
		c.Ctx.Output.Header("Access-Control-Allow-Credentials", allowCredentials)
	}
}
